@@ -52,11 +52,11 @@ import java.lang.reflect.Member;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.net.URL;
+import java.security.AccessController;
 import java.security.AllPermission;
 import java.security.Permissions;
-import java.security.ProtectionDomain;
-import java.security.AccessController;
 import java.security.PrivilegedAction;
+import java.security.ProtectionDomain;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
@@ -181,19 +181,12 @@ public final class Class implements Serializable
   /** The class signers. */
   private Object[] signers;
   /** The class protection domain. */
-  private ProtectionDomain pd;
+  private final ProtectionDomain pd;
 
-  /*
-     SableVM restriction:
-
-     java.lang.Class cannot have static initializers.  But, nothing
-     prevents an inner class to have them and achieve the equivalent
-     semantics.
-  */
-  private static class StaticData
+  /* We use an inner class, so that Class doesn't have a static initializer */
+  private static final class StaticData
   {
-    /** The unknown protection domain. */
-    private final static ProtectionDomain unknownProtectionDomain;
+    static final ProtectionDomain unknownProtectionDomain;
 
     static
     {
@@ -203,7 +196,7 @@ public final class Class implements Serializable
     }
   }
 
-  transient final Object vmdata;
+  final transient Object vmdata;
 
   /** newInstance() caches the default constructor */
   private transient Constructor constructor;
@@ -212,9 +205,19 @@ public final class Class implements Serializable
    * Class is non-instantiable from Java code; only the VM can create
    * instances of this class.
    */
-  Class(Object vmdata)
+
+  Class(Object vmdata, ProtectionDomain pd)
   {
     this.vmdata = vmdata;
+    this.pd = pd;
+    // If the VM didn't supply a protection domain and the class is an array,
+    // we "inherit" the protection domain from the component type class. This
+    // saves the VM from having to worry about protection domains for array
+    // classes.
+    //if (pd == null && isArray())
+    //  this.pd = getComponentType().pd;
+    //else
+    //  this.pd = pd;
   }
 
 
@@ -224,6 +227,7 @@ public final class Class implements Serializable
     this.primitiveType = primitiveType;
     isPrimitive = true;
     this.vmdata = null;
+    this.pd = null;
   }
   char primitiveType;
 
@@ -356,7 +360,7 @@ public final class Class implements Serializable
     if (classloader == null)
       {
         // Check if we may access the bootstrap classloader
-        SecurityManager sm = System.getSecurityManager();
+        SecurityManager sm = SecurityManager.current;
         if (sm != null)
           {
             // Get the calling class and classloader
@@ -420,8 +424,8 @@ public final class Class implements Serializable
   /**
    * Get the ClassLoader that loaded this class.  If the class was loaded
    * by the bootstrap classloader, this method will return null.
-   * If there is a security manager, and the caller's class loader does not
-   * match the requested one, a security check of
+   * If there is a security manager, and the caller's class loader is not
+   * an ancestor of the requested one, a security check of
    * <code>RuntimePermission("getClassLoader")</code>
    * must first succeed. Primitive types and void return null.
    *
@@ -432,27 +436,24 @@ public final class Class implements Serializable
    */
   public ClassLoader getClassLoader()
   {
-    // Check some common cases.
     if (isPrimitive())
       return null;
-    String name = getName();
-    if (name.startsWith("java.") || name.startsWith("gnu.java."))
-      return null;
 
     ClassLoader loader = VMClass.getClassLoader(this);
     // Check if we may get the classloader
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
         // Get the calling class and classloader
         Class c = VMSecurityManager.getClassContext()[1];
-        ClassLoader cl = c.getClassLoader();
-        if (cl != null && cl != ClassLoader.systemClassLoader)
+        ClassLoader cl = VMClass.getClassLoader(c);
+        if (cl != null && !cl.isAncestorOf(loader))
           sm.checkPermission(new RuntimePermission("getClassLoader"));
       }
+
     return loader;
   }
-  
+
   /**
    * If this is an array, get the Class representing the type of array.
    * Examples: "[[Ljava.lang.String;" would return "[Ljava.lang.String;", and
@@ -1132,8 +1133,16 @@ public final class Class implements Serializable
 
   /**
    * Get the name of this class, separated by dots for package separators.
-   * Primitive types and arrays are encoded as:
+   * If the class represents a primitive type, or void, then the
+   * name of the type as it appears in the Java programming language
+   * is returned.  For instance, <code>Byte.TYPE.getName()</code>
+   * returns "byte".
+   *
+   * Arrays are specially encoded as shown on this table.
    * <pre>
+   * array type          [<em>element type</em>
+   *                     (note that the element type is encoded per
+   *                      this table)
    * boolean             Z
    * byte                B
    * char                C
@@ -1143,7 +1152,6 @@ public final class Class implements Serializable
    * float               F
    * double              D
    * void                V
-   * array type          [<em>element type</em>
    * class or interface, alone: &lt;dotted name&gt;
    * class or interface, as element type: L&lt;dotted name&gt;;
    * </pre>
@@ -1518,7 +1526,7 @@ public final class Class implements Serializable
    */
   public ProtectionDomain getProtectionDomain()
   {
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       sm.checkPermission(new RuntimePermission("getProtectionDomain"));
 
@@ -1570,7 +1578,8 @@ public final class Class implements Serializable
         }
     else
       {
-        status = ClassLoader.systemClassAssertionStatus.get(getName());
+        status = ClassLoader.StaticData.
+                    systemClassAssertionStatus.get(getName());
         if (status != null)
           return status.equals(Boolean.TRUE);
       }
@@ -1594,11 +1603,13 @@ public final class Class implements Serializable
       {
         String name = getPackagePortion(getName());
         if ("".equals(name))
-          status = ClassLoader.systemPackageAssertionStatus.get(null);
+          status = ClassLoader.StaticData.
+                    systemPackageAssertionStatus.get(null);
         else
           do
             {
-              status = ClassLoader.systemPackageAssertionStatus.get(name);
+              status = ClassLoader.StaticData.
+                        systemPackageAssertionStatus.get(name);
               name = getPackagePortion(name);
             }
           while (! "".equals(name) && status == null);
@@ -1654,7 +1665,7 @@ public final class Class implements Serializable
    */
   private void memberAccessCheck(int which)
   {
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
 	sm.checkMemberAccess(this, which);
