@@ -1,5 +1,5 @@
 /* ClassLoader.java -- responsible for loading classes into the VM
-   Copyright (C) 1998, 1999, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
 Modifications Copyright (C) 2004 by Etienne Gagnon.
 Modifications Copyright (C) 2004 by David Belanger.
@@ -42,23 +42,27 @@ exception statement from your version. */
 
 package java.lang;
 
+import gnu.classpath.SystemProperties;
 import gnu.java.util.DoubleEnumeration;
 import gnu.java.util.EmptyEnumeration;
 
+import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
+import java.net.URLClassLoader;
 import java.security.CodeSource;
 import java.security.PermissionCollection;
 import java.security.Policy;
 import java.security.ProtectionDomain;
+import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
 import java.util.HashSet;
-import gnu.java.lang.SystemClassLoader;
 import java.lang.reflect.Constructor;
+import java.util.StringTokenizer;
 
 /**
  * The ClassLoader is a way of customizing the way Java gets its classes
@@ -87,8 +91,8 @@ import java.lang.reflect.Constructor;
  * <p>The bootstrap classloader in GNU Classpath is implemented as a couple of
  * static (native) methods on the package private class
  * <code>java.lang.VMClassLoader</code>, the system classloader is an
- * instance of <code>gnu.java.lang.SystemClassLoader</code>
- * (which is a subclass of <code>java.net.URLClassLoader</code>).
+ * anonymous inner class of ClassLoader and a subclass of
+ * <code>java.net.URLClassLoader</code>.
  *
  * <p>Users of a <code>ClassLoader</code> will normally just use the methods
  * <ul>
@@ -124,23 +128,19 @@ import java.lang.reflect.Constructor;
  */
 public abstract class ClassLoader
 {
-
-  final static Object lock = VMClassLoader.getClassLoadingLock();
-  
-
   /**
    * All classes loaded by this classloader. VM's may choose to implement
    * this cache natively; but it is here available for use if necessary. It
    * is not private in order to allow native code (and trusted subclasses)
    * access to this field.
    */
-  final Map loadedClasses = new HashMap();
+  final HashMap loadedClasses = new HashMap();
 
   /**
    * All packages defined by this classloader. It is not private in order to
    * allow native code (and trusted subclasses) access to this field.
    */
-  final Map definedPackages = new HashMap();
+  final HashMap definedPackages = new HashMap();
 
   /**
    * The classloader that is consulted before this classloader.
@@ -157,28 +157,72 @@ public abstract class ClassLoader
    */
   //private final boolean initialized;
 
-  /**
-   * System/Application classloader: defaults to an instance of
-   * gnu.java.lang.SystemClassLoader, unless the first invocation of
-   * getSystemClassLoader loads another class loader because of the
-   * java.system.class.loader property. The initialization of this field
-   * is somewhat circular - getSystemClassLoader() checks whether this
-   * field is null in order to bypass a security check.
-   */
-  //  static final ClassLoader systemClassLoader = VMClassLoader.getSystemClassLoader();
-  static final ClassLoader systemClassLoader = getSystemClassLoader();
-  
-
-  /**
-   * The default protection domain, used when defining a class with a null
-   * paramter for the domain.
-   */
-  static final ProtectionDomain defaultProtectionDomain;
-  static
+  static class StaticData
   {
-    CodeSource cs = new CodeSource(null, null);
-    PermissionCollection perm = Policy.getPolicy().getPermissions(cs);
-    defaultProtectionDomain = new ProtectionDomain(cs, perm);
+    /**
+     * The System Class Loader (a.k.a. Application Class Loader). The one
+     * returned by ClassLoader.getSystemClassLoader.
+     */
+    static final ClassLoader systemClassLoader =
+                              VMClassLoader.getSystemClassLoader();
+    static
+    {
+      // Find out if we have to install a default security manager. Note that
+      // this is done here because we potentially need the system class loader
+      // to load the security manager and note also that we don't need the
+      // security manager until the system class loader is created.
+      // If the runtime chooses to use a class loader that doesn't have the
+      // system class loader as its parent, it is responsible for setting
+      // up a security manager before doing so.
+      String secman = SystemProperties.getProperty("java.security.manager");
+      if (secman != null && SecurityManager.current == null)
+        {
+          if (secman.equals("") || secman.equals("default"))
+	    {
+	      SecurityManager.current = new SecurityManager();
+	    }
+	  else
+	    {
+	      try
+	        {
+	  	  Class cl = Class.forName(secman, false, StaticData.systemClassLoader);
+		  SecurityManager.current = (SecurityManager)cl.newInstance();
+	        }
+	      catch (Exception x)
+	        {
+		  throw (InternalError)
+		      new InternalError("Unable to create SecurityManager")
+		  	  .initCause(x);
+	        }
+	    }
+        }
+    }
+
+    /**
+     * The default protection domain, used when defining a class with a null
+     * parameter for the domain.
+     */
+    static final ProtectionDomain defaultProtectionDomain;
+    static
+    {
+        CodeSource cs = new CodeSource(null, null);
+        PermissionCollection perm = Policy.getPolicy().getPermissions(cs);
+        defaultProtectionDomain = new ProtectionDomain(cs, perm);
+    }
+    /**
+     * The command-line state of the package assertion status overrides. This
+     * map is never modified, so it does not need to be synchronized.
+     */
+    // Package visible for use by Class.
+    static final Map systemPackageAssertionStatus
+      = VMClassLoader.packageAssertionStatus();
+    /**
+     * The command-line state of the class assertion status overrides. This
+     * map is never modified, so it does not need to be synchronized.
+     */
+    // Package visible for use by Class.
+    static final Map systemClassAssertionStatus
+      = VMClassLoader.classAssertionStatus();
   }
 
   /**
@@ -188,14 +232,6 @@ public abstract class ClassLoader
   // Package visible for use by Class.
   boolean defaultAssertionStatus = VMClassLoader.defaultAssertionStatus();
 
-  /**
-   * The command-line state of the package assertion status overrides. This
-   * map is never modified, so it does not need to be synchronized.
-   */
-  // Package visible for use by Class.
-  static final Map systemPackageAssertionStatus
-    = VMClassLoader.packageAssertionStatus();
-
   /**
    * The map of package assertion status overrides, or null if no package
    * overrides have been specified yet. The values of the map should be
@@ -205,14 +241,6 @@ public abstract class ClassLoader
   // Package visible for use by Class.
   Map packageAssertionStatus;
 
-  /**
-   * The command-line state of the class assertion status overrides. This
-   * map is never modified, so it does not need to be synchronized.
-   */
-  // Package visible for use by Class.
-  static final Map systemClassAssertionStatus
-    = VMClassLoader.classAssertionStatus();
-
   /**
    * The map of class assertion status overrides, or null if no class
    * overrides have been specified yet. The values of the map should be
@@ -230,8 +258,7 @@ public abstract class ClassLoader
    */
   protected ClassLoader() throws SecurityException
   {
-    //this(systemClassLoader);
-    this(getSystemClassLoader());
+    this(StaticData.systemClassLoader);
   }
 
   /**
@@ -251,7 +278,7 @@ public abstract class ClassLoader
   protected ClassLoader(ClassLoader parent)
   {
     // May we create a new classloader?
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       sm.checkCreateClassLoader();
     this.parent = parent;
@@ -300,7 +327,7 @@ public abstract class ClassLoader
     throws ClassNotFoundException
   {
 
-    synchronized(lock)
+    synchronized(this)
       {
 	if (name.charAt(0) == '[')
 	  {
@@ -343,7 +370,7 @@ public abstract class ClassLoader
   final Class createArray(String name, boolean resolve)
     throws ClassNotFoundException
   {
-    synchronized(lock)
+    synchronized(this)
       {
 	if (name.charAt(0) != '[')
 	  {
@@ -445,6 +472,10 @@ public abstract class ClassLoader
 		throw new NoClassDefFoundError(name);
 	      }
 	  }
+        else if(cl != this)
+          {
+            return cl.createArray(name, resolve);
+          }
 	else
 	  {
 	    c = cl.defineArray(name, element);
@@ -470,12 +501,12 @@ public abstract class ClassLoader
     
     if (loadedClasses.get(name) != null)
       {
-	throw new LinkageError();
+	throw new LinkageError(name);
       }
     
     if (partiallyLoadedClasses.contains(name))
       {
-	throw new ClassCircularityError();
+	throw new ClassCircularityError(name);
       }
     
     partiallyLoadedClasses.add(name);
@@ -624,18 +655,16 @@ public abstract class ClassLoader
     
     if (loadedClasses.get(name) != null)
       {
-	throw new LinkageError();
+	throw new LinkageError(name);
       }
     
     if (partiallyLoadedClasses.contains(name))
       {
-	throw new ClassCircularityError();
+	throw new ClassCircularityError(name);
       }
 
     if (domain == null)
-      {
-	domain = defaultProtectionDomain;
-      }
+      domain = StaticData.defaultProtectionDomain;
 
     partiallyLoadedClasses.add(name);
     try
@@ -679,7 +708,7 @@ public abstract class ClassLoader
   protected final Class findSystemClass(String name)
     throws ClassNotFoundException
   {
-    return Class.forName(name, false, getSystemClassLoader());
+    return Class.forName(name, false, StaticData.systemClassLoader);
   }
 
   /**
@@ -695,7 +724,7 @@ public abstract class ClassLoader
   public final ClassLoader getParent()
   {
     // Check if we may return the parent classloader.
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
         Class c = VMSecurityManager.getClassContext()[1];
@@ -841,7 +870,7 @@ public abstract class ClassLoader
    */
   public static final URL getSystemResource(String name)
   {
-    return getSystemClassLoader().getResource(name);
+    return StaticData.systemClassLoader.getResource(name);
   }
 
   /**
@@ -857,7 +886,7 @@ public abstract class ClassLoader
    */
   public static Enumeration getSystemResources(String name) throws IOException
   {
-    return getSystemClassLoader().getResources(name);
+    return StaticData.systemClassLoader.getResources(name);
   }
 
   /**
@@ -912,18 +941,18 @@ public abstract class ClassLoader
 
   /**
    * Returns the system classloader. The system classloader (also called
-   * the application classloader) is the classloader that was used to
+   * the application classloader) is the classloader that is used to
    * load the application classes on the classpath (given by the system
    * property <code>java.class.path</code>. This is set as the context
    * class loader for a thread. The system property
    * <code>java.system.class.loader</code>, if defined, is taken to be the
    * name of the class to use as the system class loader, which must have
-   * a public constructor which takes a ClassLoader as a parent; otherwise this
-   * uses gnu.java.lang.SystemClassLoader.
+   * a public constructor which takes a ClassLoader as a parent. The parent
+   * class loader passed in the constructor is the default system class
+   * loader.
    *
    * <p>Note that this is different from the bootstrap classloader that
-   * actually loads all the real "system" classes (the bootstrap classloader
-   * is the parent of the returned system classloader).
+   * actually loads all the real "system" classes.
    *
    * <p>A security check will be performed for
    * <code>RuntimePermission("getClassLoader")</code> if the calling class
@@ -940,46 +969,16 @@ public abstract class ClassLoader
     // This method is called as the initialization of systemClassLoader,
     // so if there is a null value, this is the first call and we must check
     // for java.system.class.loader.
-    if (systemClassLoader == null)
-      {
-	String loader = System.getProperty("java.system.class.loader",
-					   "gnu.java.lang.SystemClassLoader");
-	try
-	  {
-	    // Give the new system class loader a null parent.
-	    Constructor c = Class.forName(loader).getConstructor
-	      ( new Class[] { ClassLoader.class } );
-	    return (ClassLoader) c.newInstance(new Object[1]);
-	  }
-	catch (Exception e)
-	  {
-	    try
-	      {
-		System.err.println("Requested system classloader "
-				   + loader + " failed, trying "
-				   + "gnu.java.lang.SystemClassLoader");
-		e.printStackTrace();
-		// Fallback to gnu.java.lang.SystemClassLoader.
-		return new SystemClassLoader(null);
-	      }
-	    catch (Exception e1)
-	      {
-		throw (Error) new InternalError
-		  ("System class loader could not be found: " + e1)
-		  .initCause(e1);
-	      }
-	  }
-      }
     // Check if we may return the system classloader
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
 	Class c = VMSecurityManager.getClassContext()[1];
 	ClassLoader cl = c.getClassLoader();
-	if (cl != null && cl != systemClassLoader)
+	if (cl != null && cl != StaticData.systemClassLoader)
 	  sm.checkPermission(new RuntimePermission("getClassLoader"));
       }
-    return systemClassLoader;
+    return StaticData.systemClassLoader;
   }
     
   /**
@@ -1138,7 +1137,7 @@ public abstract class ClassLoader
   {
     if (packageAssertionStatus == null)
       packageAssertionStatus
-        = new HashMap(systemPackageAssertionStatus);
+        = new HashMap(StaticData.systemPackageAssertionStatus);
     packageAssertionStatus.put(name, Boolean.valueOf(enabled));
   }
   
@@ -1158,7 +1157,8 @@ public abstract class ClassLoader
                                                    boolean enabled)
   {
     if (classAssertionStatus == null)
-      classAssertionStatus = new HashMap(systemClassAssertionStatus);
+      classAssertionStatus = 
+        new HashMap(StaticData.systemClassAssertionStatus);
     // The toString() hack catches null, as required.
     classAssertionStatus.put(name.toString(), Boolean.valueOf(enabled));
   }
@@ -1195,4 +1195,110 @@ public abstract class ClassLoader
       }
     return false;
   }
+
+  private static URL[] getExtClassLoaderUrls()
+  {
+    String classpath = SystemProperties.getProperty("java.ext.dirs", "");
+    StringTokenizer tok = new StringTokenizer(classpath, File.pathSeparator);
+    ArrayList list = new ArrayList();
+    while (tok.hasMoreTokens())
+      {
+	try
+	  {
+	    File f = new File(tok.nextToken());
+	    File[] files = f.listFiles();
+	    for (int i = 0; i < files.length; i++)
+	      {
+		list.add(files[i].toURL());
+	      }
+	  }
+	catch(Exception x)
+	  {
+	  }
+      }
+    URL[] urls = new URL[list.size()];
+    list.toArray(urls);
+    return urls;
+  }
+
+  private static void addFileURL(ArrayList list, String file)
+  {
+    try
+      {
+	list.add(new File(file).toURL());
+      }
+    catch(java.net.MalformedURLException x)
+      {
+      }
+  }
+
+  private static URL[] getSystemClassLoaderUrls()
+  {
+    String classpath = SystemProperties.getProperty("java.class.path", ".");
+    StringTokenizer tok = new StringTokenizer(classpath, File.pathSeparator, true);
+    ArrayList list = new ArrayList();
+    while (tok.hasMoreTokens())
+      {
+	String s = tok.nextToken();
+	if (s.equals(File.pathSeparator))
+	    addFileURL(list, ".");
+	else
+	  {
+	    addFileURL(list, s);
+	    if (tok.hasMoreTokens())
+	      {
+		// Skip the separator.
+		tok.nextToken();
+		// If the classpath ended with a separator,
+		// append the current directory.
+		if (!tok.hasMoreTokens())
+		    addFileURL(list, ".");
+	      }
+	  }
+      }
+    URL[] urls = new URL[list.size()];
+    list.toArray(urls);
+    return urls;
+  }
+
+  static ClassLoader defaultGetSystemClassLoader()
+  {
+    ClassLoader extClassLoader =
+	new URLClassLoader(getExtClassLoaderUrls(), null);
+    ClassLoader systemClassLoader =
+	new URLClassLoader(getSystemClassLoaderUrls(), extClassLoader)
+	{
+	    protected synchronized Class loadClass(String name,
+		boolean resolve)
+		throws ClassNotFoundException
+	    {
+		SecurityManager sm = SecurityManager.current;
+		if (sm != null)
+		{
+		    int lastDot = name.lastIndexOf('.');
+		    if (lastDot != -1)
+			sm.checkPackageAccess(name.substring(0, lastDot));
+		}
+		return super.loadClass(name, resolve);
+	    }
+	};
+    String loader = SystemProperties.getProperty("java.system.class.loader", null);
+    if (loader == null)
+      {
+	return systemClassLoader;
+      }
+    try
+      {
+	Constructor c = Class.forName(loader, false, systemClassLoader)
+	    .getConstructor(new Class[] { ClassLoader.class });
+	return (ClassLoader)c.newInstance(new Object[] { systemClassLoader });
+      }
+    catch (Exception e)
+      {
+	System.err.println("Requested system classloader " + loader + " failed.");
+	throw (Error)
+	    new Error("Requested system classloader " + loader + " failed.")
+		.initCause(e);
+      }
+  }
 }
