@@ -42,10 +42,11 @@ exception statement from your version. */
 
 package java.lang;
 
+
 /* Written using "Java Class Libraries", 2nd edition, ISBN 0-201-31002-3
  * "The Java Language Specification", ISBN 0-201-63451-1
  * plus online API docs for JDK 1.2 beta from http://www.javasoft.com.
- * Status:  Believed complete to version 1.4, with caveats. We do not 
+ * Status:  Believed complete to version 1.4, with caveats. We do not
  *          implement the deprecated (and dangerous) stop, suspend, and resume
  *          methods. Security implementation is not complete.
  */
@@ -80,7 +81,7 @@ package java.lang;
  *
  * @author Tom Tromey
  * @author John Keiser
- * @author Eric Blake <ebb9@email.byu.edu>
+ * @author Eric Blake (ebb9@email.byu.edu)
  * @see Runnable
  * @see Runtime#exit(int)
  * @see #run()
@@ -320,7 +321,7 @@ public class Thread implements Runnable
   public Thread(ThreadGroup group, Runnable target, String name, long size)
   {
     // Bypass System.getSecurityManager, for bootstrap efficiency.
-    SecurityManager sm = Runtime.securityManager;
+    SecurityManager sm = SecurityManager.current;
     Thread current = currentThread();
     if (group == null)
       {
@@ -416,7 +417,7 @@ public class Thread implements Runnable
   public final void checkAccess()
   {
     // Bypass System.getSecurityManager, for bootstrap efficiency.
-    SecurityManager sm = Runtime.securityManager;
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       sm.checkAccess(this);
   }
@@ -532,9 +533,16 @@ public class Thread implements Runnable
    * @throws NullPointerException if obj is null
    * @since 1.4
    */
-  public static boolean holdsLock(Object obj)
+  public static boolean holdsLock(Object obj) throws NullPointerException
   {
-    return VMThread.holdsLock(obj);
+    if (obj != null)
+    {
+      return VMThread.holdsLock(obj);
+    }
+    else
+    {
+      throw new NullPointerException();
+    }
   }
 
   /**
@@ -734,7 +742,7 @@ public class Thread implements Runnable
   public synchronized ClassLoader getContextClassLoader()
   {
     // Bypass System.getSecurityManager, for bootstrap efficiency.
-    SecurityManager sm = Runtime.securityManager;
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       // XXX Don't check this if the caller's class loader is an ancestor.
       sm.checkPermission(new RuntimePermission("getClassLoader"));
@@ -755,7 +763,7 @@ public class Thread implements Runnable
    */
   public synchronized void setContextClassLoader(ClassLoader classloader)
   {
-    SecurityManager sm = System.getSecurityManager();
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       sm.checkPermission(new RuntimePermission("setContextClassLoader"));
     this.contextClassLoader = classloader;
@@ -800,7 +808,7 @@ public class Thread implements Runnable
    * are no guarantees which thread will be next to run, but most VMs will
    * choose the highest priority thread that has been waiting longest.
    *
-   * @param ms the number of milliseconds to sleep, or 0 for forever
+   * @param ms the number of milliseconds to sleep
    * @throws InterruptedException if the Thread is interrupted; it's
    *         <i>interrupted status</i> will be cleared
    * @see #notify()
@@ -823,7 +831,7 @@ public class Thread implements Runnable
    * because some other thread may be active.  So don't expect real-time
    * performance.
    *
-   * @param ms the number of milliseconds to sleep, or 0 for forever
+   * @param ms the number of milliseconds to sleep
    * @param ns the number of extra nanoseconds to sleep (0-999999)
    * @throws InterruptedException if the Thread is interrupted; it's
    *         <i>interrupted status</i> will be cleared
@@ -833,6 +841,9 @@ public class Thread implements Runnable
    */
   public static void sleep(long ms, int ns) throws InterruptedException
   {
+    if(ms < 0 || ns < 0 || ns > 999999)
+	throw new IllegalArgumentException();
+
     VMThread.sleep(ms, ns);
   }
 
@@ -887,7 +898,7 @@ public class Thread implements Runnable
    * If you stop a Thread that has not yet started, the stop is ignored
    * (contrary to what the JDK documentation says).
    * <b>WARNING</b>This bypasses Java security, and can throw a checked
-   * exception which the call stack is unprepared to handle. Do not abuse 
+   * exception which the call stack is unprepared to handle. Do not abuse
    * this power.
    *
    * <p>This is inherently unsafe, as it can interrupt synchronized blocks and
@@ -915,7 +926,7 @@ public class Thread implements Runnable
     if (t == null)
       throw new NullPointerException();
     // Bypass System.getSecurityManager, for bootstrap efficiency.
-    SecurityManager sm = Runtime.securityManager;
+    SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
         sm.checkAccess(this);
