@@ -57,7 +57,6 @@ Systems    : all
 #include "config.h"
 
 #include <stdlib.h>
-#include <assert.h>
 
 #ifdef __APPLE__
 typedef unsigned int socklen_t;
@@ -420,7 +419,6 @@ typedef unsigned int socklen_t;
       result=(getsockname(socketDescriptor,(struct sockaddr*)&__socketAddress,&__socketAddressLength)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__socketAddressLength>=sizeof(__socketAddress)); \
         localAddress=ntohl(__socketAddress.sin_addr.s_addr); \
         localPort   =ntohs(__socketAddress.sin_port); \
       } \
@@ -455,7 +453,6 @@ typedef unsigned int socklen_t;
       result=(getpeername(socketDescriptor,(struct sockaddr*)&__socketAddress,&__socketAddressLength)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__socketAddressLength>=sizeof(__socketAddress)); \
         remoteAddress=ntohl(__socketAddress.sin_addr.s_addr); \
         remotePort   =ntohs(__socketAddress.sin_port); \
       } \
@@ -944,7 +941,6 @@ typedef unsigned int socklen_t;
       result=(getsockopt(socketDescriptor,IPPROTO_TCP,TCP_NODELAY,&__value,&__len)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__len>=sizeof(__value)); \
         flag=__value; \
       } \
     } while (0)
@@ -978,7 +974,6 @@ typedef unsigned int socklen_t;
       result=(getsockopt(socketDescriptor,SOL_SOCKET,SO_LINGER,&__linger,&__len)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__len>=sizeof(__linger)); \
         flag =__linger.l_onoff; \
         value=__linger.l_linger; \
       } \
@@ -1011,7 +1006,6 @@ typedef unsigned int socklen_t;
       result=(getsockopt(socketDescriptor,SOL_SOCKET,SO_TIMEOUT,&__value,&__len)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__len>=sizeof(__value)); \
         flag=__value; \
       } \
     } while (0)
@@ -1043,7 +1037,6 @@ typedef unsigned int socklen_t;
       result=(getsockopt(socketDescriptor,SOL_SOCKET,SO_SNDBUF,&__value,&__len)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__len>=sizeof(__value)); \
         size=__value; \
       } \
     } while (0)
@@ -1075,7 +1068,6 @@ typedef unsigned int socklen_t;
       result=(getsockopt(socketDescriptor,SOL_SOCKET,SO_RCVBUF,&__value,&__len)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__len>=sizeof(__value)); \
         size=__value; \
       } \
     } while (0)
@@ -1108,7 +1100,6 @@ typedef unsigned int socklen_t;
       result=(getsockopt(socketDescriptor,IPPROTO_IP,IP_TTL,&__value,&__len)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__len>=sizeof(__value)); \
         flag=__value; \
       } \
     } while (0)
@@ -1144,7 +1135,6 @@ typedef unsigned int socklen_t;
       result=(getsockopt(socketDescriptor,IPPROTO_IP,IP_MULTICAST_IF,&__socketAddress,&__socketAddressLength)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__socketAddressLength>=sizeof(__socketAddress)); \
         address=ntohl(__socketAddress.sin_addr.s_addr); \
       } \
     } while (0)
@@ -1178,7 +1168,6 @@ typedef unsigned int socklen_t;
       result=(getsockname(socketDescriptor,(struct sockaddr*)&__socketAddress,&__socketAddressLength)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__socketAddressLength>=sizeof(__socketAddress)); \
         address=ntohl(__socketAddress.sin_addr.s_addr); \
       } \
     } while (0)
@@ -1211,7 +1200,6 @@ typedef unsigned int socklen_t;
       result=(getsockopt(socketDescriptor,SOL_SOCKET,SO_REUSEADDR,&__value,&__len)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__len>=sizeof(__value)); \
         flag=__value; \
       } \
     } while (0)
@@ -1244,7 +1232,6 @@ typedef unsigned int socklen_t;
       result=(getsockopt(socketDescriptor,SOL_SOCKET,SO_KEEPALIVE,&__value,&__len)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__len>=sizeof(__value)); \
         flag=__value; \
       } \
     } while (0)
@@ -1277,7 +1264,6 @@ typedef unsigned int socklen_t;
       result=(getsockopt(socketDescriptor,SOL_SOCKET,SO_BROADCAST,&__value,&__len)==0)?TARGET_NATIVE_OK:TARGET_NATIVE_ERROR; \
       if (result==TARGET_NATIVE_OK) \
       { \
-        assert(__len>=sizeof(__value)); \
         flag=__value; \
       } \
     } while (0)
